Connection

Struct Connection 

pub struct Connection { /* private fields */ }
Connection handles a single AGI stream (a connection originating from a client). commands are sent with connection::Connection::send_command

Implementations

Source

impl Connection

pub async fn send_command<H>( &mut self, command: H, ) -> Result<AGIResponse<H::Response>, AGIError>
where H: AGICommand,

Send an AGI Command over this connection.

Return an Error when sending fails or we do not get a Status message as a response. non-200 status codes are returned as Ok(the-status) and are NOT an Err as far as this method is concerned.

Note that the precice return type depends on the command sent.